// A panel in which to display redirected stdin/stdout/stderr.
// 18 Aug 1997 Derek Lieber
//
package jd;
import ui.*;
import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;

public
class ConsolePanel extends FastPanel
   implements Constants
   {
   private Container  titlebar;
   private MultiLabel label;
   private Console    console;
   private Vector     lineBuf;

   // Create console with title bar.
   //
   ConsolePanel(String title)
      {
      titlebar = new FastPanel();
      label    = new MultiLabel(title);
      console  = new Console();
      lineBuf  = new Vector();
      
      titlebar.setLayout(new BorderLayout());
      titlebar.add("Center", label);

      setLayout(new BorderLayout());
      add("North", titlebar);
      add("Center", console);
      
      console.setFont(MONOSPACED_FONT);

      titlebar.addMouseListener(new MouseAdapter()
         { 
         public void mouseEntered(MouseEvent  e) { TextPanel.mouseEnteredOrMovedOn(ConsolePanel.this); }
         public void mouseClicked(MouseEvent  e) { Session.eventQueue.put(new DebuggerEvent(DebuggerEvent.ZOOM, ConsolePanel.this)); }
         });

      console.addMouseMotionListener(new MouseMotionAdapter()
         { public void mouseMoved(MouseEvent e) { TextPanel.mouseEnteredOrMovedOn(ConsolePanel.this); }
         });

  //  setCursor(console.getCursor());
  //  titlebar.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
      }
      
   // Fetch a "cooked" character from the console input area.
   // Cooking means: 
   //    If necessary, first wait for a complete line to be entered.
   //    The line is echoed on the console output area as it is typed.
   //    The backspace key may be used for line editing.
   //    The enter key is converted to '\n'.
   //    All other control characters are rejected.
   //    The line is complete when the "enter" key is typed.
   //
   synchronized
   char
   get()
      {
      if (lineBuf.size() == 0)
         {
         loop: for (;;)
            {
            char ch = console.get();
            switch (ch)
               {
               case '\b': 
               if (lineBuf.size() == 0) 
                  console.beep();
               else
                  {
                  console.bsp();
                  lineBuf.removeElementAt(lineBuf.size() - 1);
                  }
               break;
               
               case '\r':
               console.put('\n');
               lineBuf.addElement(new Character('\n'));
               break loop;
               
               default:
               if (ch < ' ' || ch > '~')
                  console.beep();
               else
                  {
                  console.put(ch);
                  lineBuf.addElement(new Character(ch));
                  }
               break;
               }
            }
         }

      char ch = ((Character)lineBuf.elementAt(0)).charValue();
      lineBuf.removeElementAt(0);
      return ch;
      }
      
   // Append a string to the console output area.
   //
   void
   put(String s)
      {
      console.put(s);
      }
      
   // Test.
   //
   public static void
   main(String args[])
      {
      Frame        f = new Frame();
      ConsolePanel c = new ConsolePanel("Test Console");

      f.addWindowListener(new WindowAdapter() { public void windowClosing(WindowEvent e) { System.exit(0); }});
      f.add("Center", c);
      f.pack();
      f.setSize(new Dimension(400, 400));
      f.setVisible(true);
      
      for (;;)
         c.put("got " + c.get() + "\n");
      }
   }

class Console extends TextArea
   {
   private boolean isEmpty = true;
   private char    keyBuf  = 0;

   // Create a console window, initially empty.
   //
   Console()
      {
      super("", 0, 0, SCROLLBARS_VERTICAL_ONLY);
      addKeyListener(new KeyAdapter() { public void keyTyped(KeyEvent e) { stroke(e.getKeyChar()); }});
      addMouseListener(new MouseAdapter() { public void mouseEntered(MouseEvent e) { requestFocus(); }});
      setEditable(false);
   // setCursor(ui.Constants.STANDARD_CURSOR);
      }

   // Sound a tone.
   //
   synchronized
   void
   beep()
      {
      Toolkit.getDefaultToolkit().beep();
      }

   // Backspace cursor and erase one character.
   //
   synchronized
   void
   bsp()
      {
      String text = getText();
      int    len  = text.length();

      if (len-- == 0)
         return;

      // remove last character of text
      setText(text.substring(0, len));

      // put cursor at end of text
      select(len - 1, len);
      insert("", len);
      }

   // Append a string and advance cursor.
   //
   synchronized
   void
   put(String s)
      {
      append(s);
      }

   // Append a character and advance cursor.
   //
   synchronized
   void
   put(char c)
      {
      append(String.valueOf(c));
      }

   // Wait for a character to be typed (don't echo it).
   //
   synchronized
   char
   get()
      {
      try 
         {
         while (isEmpty)
            wait();
         isEmpty = true;
         return keyBuf;
         }
      catch (InterruptedException e)
         {
         isEmpty = true;
         return 0;
         }
      }

   // Internal: process a keystroke.
   //
   synchronized
   /* private */ void
   stroke(char c)
      {
      if (isEmpty)
         { // space available
         keyBuf  = c;
         isEmpty = false;
         notify();
         }
      else
         { // overflow
         beep();
         }
      }
   }
