// Events reported to debugger by window-event-handler and debug-event-handler threads.
// 11 Mar 1996 Derek Lieber
//
package jd;
import  sun.tools.debug.*;
import  java.awt.Component;

class DebuggerEvent
   {
   // local (window) events
   //
   static final int BUTTON_SOURCEPATH   =  0;
   static final int BUTTON_CLASSPATH    =  1;
   static final int BUTTON_SHUTDOWN     =  2;
   static final int BUTTON_RESTART      =  3;
   static final int BUTTON_STOP         =  4;
   static final int BUTTON_GO           =  5;
   static final int BUTTON_SNAPSHOT     =  6;
   static final int BUTTON_STEP_OVER    =  7;
   static final int BUTTON_STEP_IN      =  8;
   static final int BUTTON_STEP_OUT     =  9;
   static final int BUTTON_RUN          = 10;
   static final int BUTTON_EDIT         = 11;
   static final int PICK                = 12;
   static final int SEARCHF             = 13;
   static final int SEARCHB             = 14;
   static final int GOTO                = 15;
   static final int ZOOM                = 16;
   
   // remote (debugging) events
   //
   static final int REMOTE_BREAKPOINT   = 20;
   static final int REMOTE_EXCEPTION    = 21;
   static final int REMOTE_THREAD_DEATH = 22;
   static final int REMOTE_QUIT         = 23;
   
   int          kind;       // kind of event       contents of field
                            // -------------       -----------------
   Object       target;     // PICK                Snapshot from which an item was picked
                            // SEARCHF             TextPanel containing string sought
                            // SEARCHB             TextPanel containing string sought
                            // GOTO                TextPanel containing line sought
                            // ZOOM                Component to be zoomed
                            // REMOTE_BREAKPOINT   RemoteThread that arrived at breakpoint
                            // REMOTE_EXCEPTION    RemoteThread that generated exception
                            // REMOTE_THREAD_DEATH RemoteThread that died

   String       text;       // PICK                line of text that was picked
                            // REMOTE_EXCEPTION    exception message and stacktrace
                          
   int          lino;       // PICK                line number that was picked
   
   int          colno;      // PICK                column number that was picked
      
   DebuggerEvent(int kind)
      {
      this.kind = kind;
      }
      
   DebuggerEvent(int kind, Snapshot pickedTarget, String pickedText, int pickedLino, int pickedColno)
      {
      this.kind   = kind;
      this.target = pickedTarget;
      this.text   = pickedText;
      this.lino   = pickedLino;
      this.colno  = pickedColno;
      }
      
   DebuggerEvent(int kind, RemoteThread thread)
      {
      this.kind   = kind;
      this.target = thread;
      }
      
   DebuggerEvent(int kind, RemoteThread thread, String message)
      {
      this.kind   = kind;
      this.target = thread;
      this.text   = message;
      }
      
   DebuggerEvent(int kind, Component zoomTarget)
      {
      this.kind   = kind;
      this.target = zoomTarget;
      }
      
   synchronized
   public String
   toString()
      {
      switch (kind)
         {
         case BUTTON_SOURCEPATH:   return "<sourcepath>";
         case BUTTON_CLASSPATH:    return "<classpath>";
         
         case BUTTON_SHUTDOWN:     return "<shutdown>";
         case BUTTON_RESTART:      return "<restart>";
         case BUTTON_STOP:         return "<stop>";
         case BUTTON_GO:           return "<go>";
         case BUTTON_RUN:          return "<run>";
         case BUTTON_STEP_OVER:    return "<step over>";
         case BUTTON_STEP_IN:      return "<step in>";
         case BUTTON_STEP_OUT:     return "<step out>";
         case BUTTON_EDIT:         return "<edit>";
         case BUTTON_SNAPSHOT:     return "<snapshot>";
         
         case PICK:                return "<pick>";
         case SEARCHF:             return "<searchf>";
         case SEARCHB:             return "<searchb>";
         case GOTO:                return "<goto>";
         case ZOOM:                return "<zoom>";
         
         case REMOTE_BREAKPOINT:   return "<remote breakpoint>";
         case REMOTE_EXCEPTION:    return "<remote exception>";
         case REMOTE_THREAD_DEATH: return "<remote thread death>";
         case REMOTE_QUIT:         return "<remote quit>";
         
         default:                  return "<debugger event " + kind + ">";
         }
      }
   }
