// Frame in which to display main panel.
// 11 Mar 1996 Derek Lieber
//
package jd;
import  java.awt.*;
import  java.awt.event.*;
import  util.Tracer;

class DebuggerFrame extends Frame
   implements Constants, WindowListener, ActionListener
   {
   Debugger      debugger;
   DebuggerPanel debuggerPanel;
   
   static Image  logoImage;
   static Image  iconImage;

   static Image  stopSignImage;
   static Image  greenLightImage;
   static Image  redLightImage;
   
   static Image  stopButtonImage;
   static Image  goButtonImage;
   static Image  refreshButtonImage;
   static Image  refreshButtonAltImage;

   static Image  stepOverButtonImage;
   static Image  stepInButtonImage;
   static Image  stepOutButtonImage;
   static Image  runButtonImage;
   
   static Image  editButtonImage;
   
   DebuggerFrame(Debugger debugger, String title)
      {
      //!!TODO: think about alternate way to access images and other resources.
      // Eg:
      //       java.net.URL url = getClass().getResource("foobar.gif");
      //
      // Yields something like:
      //
      //       "systemresource:/ZIPd:\derek\jbin\jd.zip/+/jd/foobar.gif"
      //        |<--------------->||<----------------->| |<---------->|
      //          type indicator         zipfile            zipentry
      //
      
      createImages(this);

      this.debugger      = debugger;
      this.debuggerPanel = new DebuggerPanel(debugger);
      
      add("Center", debuggerPanel);
      addWindowListener(this);

      // choose title, color, font, icon
      //
      setTitle("Jd " + title);
      setBackground(SystemColor.text);
      setForeground(SystemColor.textText);
      setFont(SERIF_FONT);
      setIconImage(iconImage);
      setBusyCursor();

      // setup menu bar
      //
      MenuBar  menuBar;
      Menu     menu;
      MenuItem menuItem;

      setMenuBar(menuBar = new MenuBar());
      menuBar.add(menu   = new Menu("Session"));
      menu.add(menuItem  = new MenuItem(EDIT_SOURCEPATH_TITLE)); menuItem.addActionListener(this);
      menu.add(menuItem  = new MenuItem(EDIT_CLASSPATH_TITLE));  menuItem.addActionListener(this);
      menu.add(menuItem  = new MenuItem(RESTART_TITLE));         menuItem.addActionListener(this);
      menu.add(menuItem  = new MenuItem(EXIT_TITLE));            menuItem.addActionListener(this);
      debuggerPanel.addMenu(menuBar);

      // initiate AWT thread
      pack(); 
      
      // place main window on desktop to match previous incarnation
      getProperties();
      
      // reveal main window
      setVisible(true);
      }

   // Change cursor shape.
   //
   void setNormalCursor() { setCursor(ui.Constants.STANDARD_CURSOR); }
   void setBusyCursor()   { setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR)); }

   // Make frame agree with property list settings.
   //
   synchronized
   void
   getProperties()
      {
      Dimension screenSize = getToolkit().getScreenSize();
      
      String    defaultX = "0";
      String    defaultY = "0";
      String    defaultW = new Integer((int)(.9 * screenSize.width )).toString();
      String    defaultH = new Integer((int)(.9 * screenSize.height)).toString();

      int x = Integer.valueOf(Session.getProperty("Frame.x",     defaultX)).intValue();
      int y = Integer.valueOf(Session.getProperty("Frame.y",     defaultY)).intValue();
      int w = Integer.valueOf(Session.getProperty("Frame.width", defaultW)).intValue();
      int h = Integer.valueOf(Session.getProperty("Frame.height",defaultH)).intValue();

      setBounds(x, y, w, h);
      }

   // Make property list settings agree with frame.
   //
   synchronized
   void
   putProperties()
      {
      Rectangle r = getBounds();
      Session.putProperty("Frame.x",      "" + r.x);
      Session.putProperty("Frame.y",      "" + r.y);
      Session.putProperty("Frame.width",  "" + r.width);
      Session.putProperty("Frame.height", "" + (r.height + 20)); // +20 because reported height of frame does not seem to...
                                                                  // ...include system-supplied title bar. sigh.
      }

   private static void
   createImages(Component component)
      {
      Toolkit      toolkit = Toolkit.getDefaultToolkit();
      MediaTracker tracker = new MediaTracker(component);

      tracker.addImage(logoImage = toolkit.createImage(new byte[]
         { // logo.gif
         (byte)0x47,(byte)0x49,(byte)0x46,(byte)0x38,(byte)0x39,(byte)0x61,(byte)0x27,(byte)0x00,
         (byte)0x0f,(byte)0x00,(byte)0xf0,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,
         (byte)0xff,(byte)0x00,(byte)0x00,(byte)0x21,(byte)0xf9,(byte)0x04,(byte)0x01,(byte)0x00,
         (byte)0x00,(byte)0x01,(byte)0x00,(byte)0x2c,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,
         (byte)0x27,(byte)0x00,(byte)0x0f,(byte)0x00,(byte)0x00,(byte)0x02,(byte)0x44,(byte)0x84,
         (byte)0x0f,(byte)0x11,(byte)0xc8,(byte)0xaa,(byte)0x9c,(byte)0xdc,(byte)0x6a,(byte)0xb2,
         (byte)0xda,(byte)0x8b,(byte)0x31,(byte)0x8c,(byte)0x1b,(byte)0x3d,(byte)0xe8,(byte)0x6c,
         (byte)0xd9,(byte)0x48,(byte)0x92,(byte)0x87,(byte)0x75,(byte)0x7e,(byte)0x46,(byte)0xe8,
         (byte)0xa9,(byte)0x6b,(byte)0x09,(byte)0xc7,(byte)0x6c,(byte)0x24,(byte)0x75,(byte)0xf4,
         (byte)0xe4,(byte)0x81,(byte)0xf2,(byte)0x6e,(byte)0xbe,(byte)0x75,(byte)0xf7,(byte)0x4b,
         (byte)0x34,(byte)0x52,(byte)0xbc,(byte)0x22,(byte)0xca,(byte)0x37,(byte)0x9b,(byte)0xd1,
         (byte)0x4e,(byte)0x43,(byte)0xa4,(byte)0xb1,(byte)0x28,(byte)0xc2,(byte)0x01,(byte)0xa5,
         (byte)0x9c,(byte)0x17,(byte)0xe5,(byte)0xc9,(byte)0x8b,(byte)0xda,(byte)0x82,(byte)0xbe,
         (byte)0x25,(byte)0xa2,(byte)0x00,(byte)0x00,(byte)0x3b
         }), 0);

      tracker.addImage(iconImage = toolkit.createImage(new byte[]
         { // smallBug.gif
         (byte)0x47,(byte)0x49,(byte)0x46,(byte)0x38,(byte)0x39,(byte)0x61,(byte)0x10,(byte)0x00,
         (byte)0x10,(byte)0x00,(byte)0xf1,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,
         (byte)0x00,(byte)0xff,(byte)0x00,(byte)0xff,(byte)0x00,(byte)0x00,(byte)0xff,(byte)0xff,
         (byte)0x00,(byte)0x21,(byte)0xf9,(byte)0x04,(byte)0x01,(byte)0x00,(byte)0x00,(byte)0x02,
         (byte)0x00,(byte)0x2c,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x10,(byte)0x00,
         (byte)0x10,(byte)0x00,(byte)0x00,(byte)0x02,(byte)0x34,(byte)0x94,(byte)0x8f,(byte)0xa9,
         (byte)0x9b,(byte)0xd0,(byte)0x8e,(byte)0xa0,(byte)0x90,(byte)0xa0,(byte)0xd2,(byte)0x78,
         (byte)0x40,(byte)0x08,(byte)0x37,(byte)0xbb,(byte)0xba,(byte)0x55,(byte)0x93,(byte)0x64,
         (byte)0x58,(byte)0x5b,(byte)0x28,(byte)0x2a,(byte)0x5a,(byte)0x30,(byte)0x0c,(byte)0xe1,
         (byte)0x33,(byte)0xb1,(byte)0xee,(byte)0x9b,(byte)0x96,(byte)0xda,(byte)0x8c,(byte)0x2e,
         (byte)0xeb,(byte)0xc9,(byte)0xe9,(byte)0xe0,(byte)0x69,(byte)0x81,(byte)0xfd,(byte)0x44,
         (byte)0x35,(byte)0x92,(byte)0xc5,(byte)0x58,(byte)0x62,(byte)0x60,(byte)0x94,(byte)0x8b,
         (byte)0x02,(byte)0x00,(byte)0x3b
         }), 0);

      tracker.addImage(stopSignImage = toolkit.createImage(new byte[]
         { // stopSign.gif
         (byte)0x47,(byte)0x49,(byte)0x46,(byte)0x38,(byte)0x39,(byte)0x61,(byte)0x0d,(byte)0x00,
         (byte)0x0d,(byte)0x00,(byte)0xf1,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,
         (byte)0xff,(byte)0x00,(byte)0x00,(byte)0xff,(byte)0xff,(byte)0xff,(byte)0x00,(byte)0x00,
         (byte)0x00,(byte)0x21,(byte)0xf9,(byte)0x04,(byte)0x01,(byte)0x00,(byte)0x00,(byte)0x00,
         (byte)0x00,(byte)0x2c,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x0d,(byte)0x00,
         (byte)0x0d,(byte)0x00,(byte)0x00,(byte)0x02,(byte)0x21,(byte)0x84,(byte)0x8f,(byte)0x69,
         (byte)0xc1,(byte)0xac,(byte)0xd0,(byte)0x44,(byte)0x4b,(byte)0x51,(byte)0x48,(byte)0xb7,
         (byte)0xea,(byte)0xc5,(byte)0x2d,(byte)0xd8,(byte)0x1d,(byte)0x64,(byte)0x51,(byte)0x17,
         (byte)0x42,(byte)0x1d,(byte)0x58,(byte)0x92,(byte)0xe8,(byte)0x41,(byte)0x76,(byte)0x14,
         (byte)0xb3,(byte)0x3d,(byte)0xe4,(byte)0x43,(byte)0x1b,(byte)0x05,(byte)0x00,(byte)0x3b
         }), 0);
      
      tracker.addImage(redLightImage = toolkit.createImage(new byte[]
         { // smallRedLight.gif
         (byte)0x47,(byte)0x49,(byte)0x46,(byte)0x38,(byte)0x39,(byte)0x61,(byte)0x09,(byte)0x00,
         (byte)0x0d,(byte)0x00,(byte)0xf0,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,
         (byte)0xff,(byte)0x00,(byte)0x00,(byte)0x21,(byte)0xf9,(byte)0x04,(byte)0x01,(byte)0x00,
         (byte)0x00,(byte)0x00,(byte)0x00,(byte)0x2c,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,
         (byte)0x09,(byte)0x00,(byte)0x0d,(byte)0x00,(byte)0x00,(byte)0x02,(byte)0x13,(byte)0x84,
         (byte)0x8f,(byte)0xa9,(byte)0x10,(byte)0xeb,(byte)0x0d,(byte)0x53,(byte)0x98,(byte)0x08,
         (byte)0xd2,(byte)0x76,(byte)0x26,(byte)0xce,(byte)0xac,(byte)0x3b,(byte)0xb5,(byte)0x7d,
         (byte)0x62,(byte)0x01,(byte)0x00,(byte)0x3b
         }), 0);

      tracker.addImage(greenLightImage = toolkit.createImage(new byte[]
         { // smallGreenLight.gif
         (byte)0x47,(byte)0x49,(byte)0x46,(byte)0x38,(byte)0x39,(byte)0x61,(byte)0x09,(byte)0x00,
         (byte)0x0d,(byte)0x00,(byte)0xf0,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,
         (byte)0x00,(byte)0x80,(byte)0x00,(byte)0x21,(byte)0xf9,(byte)0x04,(byte)0x01,(byte)0x00,
         (byte)0x00,(byte)0x00,(byte)0x00,(byte)0x2c,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,
         (byte)0x09,(byte)0x00,(byte)0x0d,(byte)0x00,(byte)0x00,(byte)0x02,(byte)0x13,(byte)0x84,
         (byte)0x8f,(byte)0xa9,(byte)0x10,(byte)0xeb,(byte)0x0d,(byte)0x53,(byte)0x98,(byte)0x08,
         (byte)0xd2,(byte)0x76,(byte)0x26,(byte)0xce,(byte)0xac,(byte)0x3b,(byte)0xb5,(byte)0x7d,
         (byte)0x62,(byte)0x01,(byte)0x00,(byte)0x3b
         }), 0);

      tracker.addImage(stopButtonImage = toolkit.createImage(new byte[]
         { // Stop.gif
         (byte)0x47,(byte)0x49,(byte)0x46,(byte)0x38,(byte)0x39,(byte)0x61,(byte)0x14,(byte)0x00,
         (byte)0x13,(byte)0x00,(byte)0xf1,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,
         (byte)0x80,(byte)0x80,(byte)0x80,(byte)0xff,(byte)0x00,(byte)0x00,(byte)0xff,(byte)0xff,
         (byte)0xff,(byte)0x21,(byte)0xf9,(byte)0x04,(byte)0x01,(byte)0x00,(byte)0x00,(byte)0x02,
         (byte)0x00,(byte)0x2c,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x14,(byte)0x00,
         (byte)0x13,(byte)0x00,(byte)0x00,(byte)0x02,(byte)0x2b,(byte)0x94,(byte)0x8f,(byte)0xa9,
         (byte)0xcb,(byte)0xed,(byte)0x0f,(byte)0xa3,(byte)0x1c,(byte)0xb4,(byte)0xd6,(byte)0xe0,
         (byte)0x06,(byte)0xd8,(byte)0x7c,(byte)0xe3,(byte)0xa6,(byte)0x75,(byte)0xdc,(byte)0xc7,
         (byte)0x84,(byte)0x22,(byte)0x40,(byte)0x2e,(byte)0xa6,(byte)0x98,(byte)0x2a,(byte)0x6b,
         (byte)0xd7,(byte)0x26,(byte)0xef,(byte)0x98,(byte)0x9d,(byte)0x9e,(byte)0x13,(byte)0xe4,
         (byte)0xfa,(byte)0x2e,(byte)0xf5,(byte)0xfe,(byte)0x0f,(byte)0x0c,(byte)0x2a,(byte)0x0a,
         (byte)0x00,(byte)0x3b
         }), 0);
      
      tracker.addImage(goButtonImage = toolkit.createImage(new byte[]
         { // Go.gif
         (byte)0x47,(byte)0x49,(byte)0x46,(byte)0x38,(byte)0x39,(byte)0x61,(byte)0x14,(byte)0x00,
         (byte)0x13,(byte)0x00,(byte)0xf1,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,
         (byte)0x84,(byte)0x82,(byte)0x84,(byte)0xff,(byte)0x00,(byte)0x00,(byte)0xff,(byte)0xff,
         (byte)0xff,(byte)0x21,(byte)0xf9,(byte)0x04,(byte)0x01,(byte)0x00,(byte)0x00,(byte)0x02,
         (byte)0x00,(byte)0x2c,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x14,(byte)0x00,
         (byte)0x13,(byte)0x00,(byte)0x00,(byte)0x02,(byte)0x28,(byte)0x94,(byte)0x8f,(byte)0xa9,
         (byte)0xcb,(byte)0xed,(byte)0x0f,(byte)0xd7,(byte)0x18,(byte)0x31,(byte)0x0d,(byte)0x30,
         (byte)0xab,(byte)0xb9,(byte)0x00,(byte)0xd3,(byte)0xc8,(byte)0x75,(byte)0x1e,(byte)0x14,
         (byte)0x8a,(byte)0x63,(byte)0x53,(byte)0x9a,(byte)0x40,(byte)0x80,(byte)0xaa,(byte)0x1d,
         (byte)0xeb,(byte)0xa4,(byte)0xf0,(byte)0x13,(byte)0xce,(byte)0xe4,(byte)0xaa,(byte)0xe5,
         (byte)0xfa,(byte)0xce,(byte)0xf7,(byte)0x4b,(byte)0x01,(byte)0x00,(byte)0x3b
         }), 0);
      
      tracker.addImage(refreshButtonImage = toolkit.createImage(new byte[]
         { // Refresh.gif
         (byte)0x47,(byte)0x49,(byte)0x46,(byte)0x38,(byte)0x39,(byte)0x61,(byte)0x14,(byte)0x00,
         (byte)0x13,(byte)0x00,(byte)0xf2,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,
         (byte)0x84,(byte)0x82,(byte)0x84,(byte)0xff,(byte)0x00,(byte)0x00,(byte)0xff,(byte)0xff,
         (byte)0x00,(byte)0xff,(byte)0xff,(byte)0xff,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,
         (byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x21,(byte)0xf9,(byte)0x04,
         (byte)0x01,(byte)0x00,(byte)0x00,(byte)0x02,(byte)0x00,(byte)0x2c,(byte)0x00,(byte)0x00,
         (byte)0x00,(byte)0x00,(byte)0x14,(byte)0x00,(byte)0x13,(byte)0x00,(byte)0x00,(byte)0x03,
         (byte)0x51,(byte)0x28,(byte)0xba,(byte)0xdc,(byte)0xce,(byte)0x81,(byte)0xbc,(byte)0x39,
         (byte)0x54,(byte)0xa8,(byte)0x22,(byte)0x4c,(byte)0x28,(byte)0x63,(byte)0xc6,(byte)0x9b,
         (byte)0x45,(byte)0x5c,(byte)0x17,(byte)0x60,(byte)0x9e,(byte)0x27,(byte)0x43,(byte)0x0c,
         (byte)0xd7,(byte)0x4a,(byte)0xbc,(byte)0x30,(byte)0x0c,(byte)0x40,(byte)0x41,(byte)0x34,
         (byte)0x10,(byte)0x28,(byte)0x8a,(byte)0xab,(byte)0x77,(byte)0x14,(byte)0x01,(byte)0x2f,
         (byte)0x20,(byte)0x6e,(byte)0xb7,(byte)0xf0,(byte)0x94,(byte)0x72,(byte)0x27,(byte)0xa2,
         (byte)0x42,(byte)0x92,(byte)0x19,(byte)0xc6,(byte)0x82,(byte)0x4a,(byte)0x06,(byte)0x10,
         (byte)0x99,(byte)0x9c,(byte)0x39,(byte)0xa6,(byte)0x4f,(byte)0xa8,(byte)0xb5,(byte)0x81,
         (byte)0xcd,(byte)0x4e,(byte)0x1f,(byte)0x53,(byte)0xea,(byte)0xf7,(byte)0x9a,(byte)0x7d,
         (byte)0x6e,(byte)0xa5,(byte)0x62,(byte)0x54,(byte)0x68,(byte)0xcd,(byte)0x6e,(byte)0xbb,
         (byte)0x15,(byte)0x09,(byte)0x00,(byte)0x3b
         }), 0);
      
      tracker.addImage(refreshButtonAltImage = toolkit.createImage(new byte[]
         { // RefreshAlt.gif
         (byte)0x47,(byte)0x49,(byte)0x46,(byte)0x38,(byte)0x39,(byte)0x61,(byte)0x14,(byte)0x00,
         (byte)0x13,(byte)0x00,(byte)0xf2,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,
         (byte)0x00,(byte)0xff,(byte)0x00,(byte)0x84,(byte)0x82,(byte)0x84,(byte)0xff,(byte)0x00,
         (byte)0x00,(byte)0xff,(byte)0x80,(byte)0x40,(byte)0xff,(byte)0xff,(byte)0xff,(byte)0x00,
         (byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x21,(byte)0xf9,(byte)0x04,
         (byte)0x01,(byte)0x00,(byte)0x00,(byte)0x03,(byte)0x00,(byte)0x2c,(byte)0x00,(byte)0x00,
         (byte)0x00,(byte)0x00,(byte)0x14,(byte)0x00,(byte)0x13,(byte)0x00,(byte)0x00,(byte)0x03,
         (byte)0x51,(byte)0x38,(byte)0xba,(byte)0xdc,(byte)0xce,(byte)0xa2,(byte)0xbc,(byte)0x49,
         (byte)0x94,(byte)0xa8,(byte)0x43,(byte)0x4c,(byte)0x28,(byte)0x63,(byte)0xc6,(byte)0x9b,
         (byte)0x55,(byte)0x5c,(byte)0x17,(byte)0x60,(byte)0x9e,(byte)0x27,(byte)0x53,(byte)0x10,
         (byte)0xd7,(byte)0x1a,(byte)0xbc,(byte)0x30,(byte)0x0c,(byte)0x40,(byte)0x42,(byte)0x44,
         (byte)0x14,(byte)0x28,(byte)0x1a,(byte)0xcc,(byte)0xcb,(byte)0x3a,(byte)0xfe,(byte)0x80,
         (byte)0x57,(byte)0x70,(byte)0xb7,(byte)0xe3,(byte)0x7c,(byte)0x72,(byte)0x3a,(byte)0x9e,
         (byte)0x42,(byte)0x92,(byte)0x21,(byte)0xc6,(byte)0x84,(byte)0x45,(byte)0x47,(byte)0x10,
         (byte)0x79,(byte)0x8a,(byte)0x36,(byte)0xa6,(byte)0x4f,(byte)0xa8,(byte)0x92,(byte)0x81,
         (byte)0xcd,(byte)0x4e,(byte)0x1f,(byte)0x53,(byte)0xea,(byte)0x57,(byte)0x9a,(byte)0x7d,
         (byte)0x6e,(byte)0x17,(byte)0xe2,(byte)0x5c,(byte)0x68,(byte)0xcd,(byte)0x6e,(byte)0xbb,
         (byte)0x15,(byte)0x09,(byte)0x00,(byte)0x3b
         }), 0);
      
      tracker.addImage(stepOverButtonImage = toolkit.createImage(new byte[]
         { // StepOver.gif
         (byte)0x47,(byte)0x49,(byte)0x46,(byte)0x38,(byte)0x39,(byte)0x61,(byte)0x14,(byte)0x00,
         (byte)0x13,(byte)0x00,(byte)0xf1,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,
         (byte)0x00,(byte)0x00,(byte)0xff,(byte)0xff,(byte)0x00,(byte)0x00,(byte)0xff,(byte)0xff,
         (byte)0xff,(byte)0x21,(byte)0xf9,(byte)0x04,(byte)0x01,(byte)0x00,(byte)0x00,(byte)0x02,
         (byte)0x00,(byte)0x2c,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x14,(byte)0x00,
         (byte)0x13,(byte)0x00,(byte)0x00,(byte)0x02,(byte)0x32,(byte)0x94,(byte)0x8f,(byte)0xa9,
         (byte)0xcb,(byte)0xed,(byte)0x0c,(byte)0xa2,(byte)0x9c,(byte)0xf2,(byte)0x59,(byte)0x33,
         (byte)0xb2,(byte)0xbe,(byte)0x62,(byte)0x84,(byte)0xff,(byte)0x71,(byte)0x1e,(byte)0x87,
         (byte)0x8c,(byte)0x06,(byte)0x45,(byte)0x25,(byte)0x26,(byte)0xd9,(byte)0x05,(byte)0xdd,
         (byte)0x80,(byte)0x4e,(byte)0xaa,(byte)0xeb,(byte)0x65,(byte)0x57,(byte)0xed,(byte)0x7d,
         (byte)0xf6,(byte)0x03,(byte)0xf6,(byte)0x2e,(byte)0x07,(byte)0x62,(byte)0x09,(byte)0x7e,
         (byte)0xa4,(byte)0x18,(byte)0x4a,(byte)0x88,(byte)0x6c,(byte)0x14,(byte)0x00,(byte)0x00,
         (byte)0x3b
         }), 0);
      
      tracker.addImage(stepInButtonImage = toolkit.createImage(new byte[]
         { // StepIn.gif
         (byte)0x47,(byte)0x49,(byte)0x46,(byte)0x38,(byte)0x39,(byte)0x61,(byte)0x14,(byte)0x00,
         (byte)0x13,(byte)0x00,(byte)0xf1,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,
         (byte)0x00,(byte)0x00,(byte)0xff,(byte)0xff,(byte)0x00,(byte)0x00,(byte)0xff,(byte)0xff,
         (byte)0xff,(byte)0x21,(byte)0xf9,(byte)0x04,(byte)0x01,(byte)0x00,(byte)0x00,(byte)0x02,
         (byte)0x00,(byte)0x2c,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x14,(byte)0x00,
         (byte)0x13,(byte)0x00,(byte)0x00,(byte)0x02,(byte)0x34,(byte)0x94,(byte)0x8f,(byte)0xa9,
         (byte)0x32,(byte)0xb3,(byte)0x7f,(byte)0x46,(byte)0x80,(byte)0x50,(byte)0x0a,(byte)0x80,
         (byte)0xb3,(byte)0xc6,(byte)0xc9,(byte)0xd2,(byte)0xe5,(byte)0x19,(byte)0x9b,(byte)0xd6,
         (byte)0x4d,(byte)0x86,(byte)0xf3,(byte)0x9d,(byte)0xa6,(byte)0x10,(byte)0x34,(byte)0xa9,
         (byte)0xd4,(byte)0x48,(byte)0x41,(byte)0x8b,(byte)0x3e,(byte)0xf2,(byte)0x8c,(byte)0xcf,
         (byte)0xe9,(byte)0x81,(byte)0xef,(byte)0xc8,(byte)0xea,(byte)0x0b,(byte)0x3e,(byte)0x46,
         (byte)0x19,(byte)0x61,(byte)0x8a,(byte)0xb8,(byte)0x31,(byte)0x2a,(byte)0x97,(byte)0x90,
         (byte)0x02,(byte)0x00,(byte)0x3b
         }), 0);

      tracker.addImage(stepOutButtonImage = toolkit.createImage(new byte[]
         { // StepOut.gif
         (byte)0x47,(byte)0x49,(byte)0x46,(byte)0x38,(byte)0x39,(byte)0x61,(byte)0x14,(byte)0x00,
         (byte)0x13,(byte)0x00,(byte)0xf1,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,
         (byte)0x00,(byte)0x00,(byte)0xff,(byte)0xff,(byte)0x00,(byte)0x00,(byte)0xff,(byte)0xff,
         (byte)0xff,(byte)0x21,(byte)0xf9,(byte)0x04,(byte)0x01,(byte)0x00,(byte)0x00,(byte)0x02,
         (byte)0x00,(byte)0x2c,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x14,(byte)0x00,
         (byte)0x13,(byte)0x00,(byte)0x00,(byte)0x02,(byte)0x2d,(byte)0x94,(byte)0x8f,(byte)0xa9,
         (byte)0xcb,(byte)0xed,(byte)0x0b,(byte)0xa2,(byte)0x9c,(byte)0xf3,(byte)0x59,(byte)0x45,
         (byte)0xb3,(byte)0x1d,(byte)0xbc,(byte)0x73,(byte)0x1b,(byte)0x84,(byte)0x62,(byte)0x78,
         (byte)0x25,(byte)0x43,(byte)0xd0,(byte)0x64,(byte)0xd2,(byte)0x59,(byte)0x22,(byte)0x6c,
         (byte)0xa2,(byte)0x4e,(byte)0x5c,(byte)0xf0,(byte)0x95,(byte)0x23,(byte)0xda,(byte)0x8a,
         (byte)0xad,(byte)0xa1,(byte)0xef,(byte)0x02,(byte)0xee,(byte)0x03,(byte)0xfa,(byte)0x86,
         (byte)0x8a,(byte)0x02,(byte)0x00,(byte)0x3b
         }), 0);
      
      tracker.addImage(runButtonImage = toolkit.createImage(new byte[]
         { // Run.gif
         (byte)0x47,(byte)0x49,(byte)0x46,(byte)0x38,(byte)0x39,(byte)0x61,(byte)0x14,(byte)0x00,
         (byte)0x13,(byte)0x00,(byte)0xf1,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,
         (byte)0x00,(byte)0x00,(byte)0xff,(byte)0xff,(byte)0x00,(byte)0x00,(byte)0xff,(byte)0xff,
         (byte)0xff,(byte)0x21,(byte)0xf9,(byte)0x04,(byte)0x01,(byte)0x00,(byte)0x00,(byte)0x02,
         (byte)0x00,(byte)0x2c,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x14,(byte)0x00,
         (byte)0x13,(byte)0x00,(byte)0x00,(byte)0x02,(byte)0x2b,(byte)0x94,(byte)0x8f,(byte)0xa9,
         (byte)0xcb,(byte)0xed,(byte)0x0f,(byte)0x11,(byte)0x98,(byte)0xb4,(byte)0xda,(byte)0x88,
         (byte)0x45,(byte)0x1d,(byte)0x43,(byte)0x57,(byte)0x38,(byte)0x04,(byte)0xd8,(byte)0x26,
         (byte)0x5e,(byte)0x4f,(byte)0xf8,(byte)0x58,(byte)0x13,(byte)0x6a,(byte)0x32,(byte)0x5c,
         (byte)0xd8,(byte)0x65,(byte)0xc1,(byte)0x3c,(byte)0x67,(byte)0x02,(byte)0x6d,(byte)0xdf,
         (byte)0x75,(byte)0x2e,(byte)0xe6,(byte)0xb7,(byte)0x0f,(byte)0x84,(byte)0x14,(byte)0x00,
         (byte)0x00,(byte)0x3b
         }), 0);
      
      tracker.addImage(editButtonImage = toolkit.createImage(new byte[]
         { // Edit.gif
         (byte)0x47,(byte)0x49,(byte)0x46,(byte)0x38,(byte)0x39,(byte)0x61,(byte)0x14,(byte)0x00,
         (byte)0x13,(byte)0x00,(byte)0xf2,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,
         (byte)0x84,(byte)0x82,(byte)0x00,(byte)0xff,(byte)0x00,(byte)0x00,(byte)0xff,(byte)0xff,
         (byte)0x00,(byte)0xff,(byte)0xff,(byte)0xff,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,
         (byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x21,(byte)0xf9,(byte)0x04,
         (byte)0x01,(byte)0x00,(byte)0x00,(byte)0x02,(byte)0x00,(byte)0x2c,(byte)0x00,(byte)0x00,
         (byte)0x00,(byte)0x00,(byte)0x14,(byte)0x00,(byte)0x13,(byte)0x00,(byte)0x00,(byte)0x03,
         (byte)0x3b,(byte)0x28,(byte)0xba,(byte)0xdc,(byte)0xfe,(byte)0x30,(byte)0x3e,(byte)0x40,
         (byte)0xa5,(byte)0x03,(byte)0x0b,(byte)0x63,(byte)0x7b,(byte)0x37,(byte)0xa3,(byte)0x5e,
         (byte)0x03,(byte)0x8a,(byte)0xc4,(byte)0x40,(byte)0x80,(byte)0x68,(byte)0x04,(byte)0x98,
         (byte)0x6c,(byte)0x59,(byte)0x86,(byte)0x64,(byte)0xdb,(byte)0xc2,(byte)0xdf,(byte)0x8c,
         (byte)0xde,(byte)0xf1,(byte)0x19,(byte)0xec,(byte)0x3c,(byte)0x0f,(byte)0xaf,(byte)0xba,
         (byte)0x5e,(byte)0x8f,(byte)0x06,(byte)0x08,(byte)0x0a,(byte)0x77,(byte)0x34,(byte)0x01,
         (byte)0xe5,(byte)0xe8,(byte)0x83,(byte)0xdc,(byte)0x9e,(byte)0x9c,(byte)0xa8,(byte)0x74,
         (byte)0x4a,(byte)0xa5,(byte)0x26,(byte)0x00,(byte)0x00,(byte)0x3b
         }), 0);

      try { tracker.waitForAll(); } catch (InterruptedException e) { }
      }
      
   // Handle menubar hits.
   //
   synchronized
   public void
   actionPerformed(ActionEvent e) //- implements ActionListener
      {
      String cmd = e.getActionCommand();
      
      if (cmd.equals(EDIT_SOURCEPATH_TITLE))
         Session.eventQueue.put(new DebuggerEvent(DebuggerEvent.BUTTON_SOURCEPATH));
      if (cmd.equals(EDIT_CLASSPATH_TITLE))
         Session.eventQueue.put(new DebuggerEvent(DebuggerEvent.BUTTON_CLASSPATH));
      if (cmd.equals(RESTART_TITLE))
         Session.eventQueue.put(new DebuggerEvent(DebuggerEvent.BUTTON_RESTART));
      if (cmd.equals(EXIT_TITLE))
         Session.eventQueue.put(new DebuggerEvent(DebuggerEvent.BUTTON_SHUTDOWN));
      }
   
   // Handle taskbar hits.
   //
   synchronized
   public void 
   windowClosing(WindowEvent e) //- implements WindowListener
      {
      Session.eventQueue.put(new DebuggerEvent(DebuggerEvent.BUTTON_SHUTDOWN));
      }

   // Nops.
   //
   public void windowOpened(WindowEvent      e) {} //- implements WindowListener
   public void windowClosed(WindowEvent      e) {} //- implements WindowListener
   public void windowIconified(WindowEvent   e) {} //- implements WindowListener
   public void windowDeiconified(WindowEvent e) {} //- implements WindowListener
   public void windowActivated(WindowEvent   e) {} //- implements WindowListener
   public void windowDeactivated(WindowEvent e) {} //- implements WindowListener
   }
