// Main display panel.
// 11 Mar 1996 Derek Lieber
//
package jd;
import  java.awt.*;
import  java.awt.event.*;
import  ui.*;

interface AdjustableContainer
   {
   // Reorganize viewing real estate to accomodate change(s) in panel visibility.
   //
   public void adjustForComponentVisibilityChange();
   }

class DebuggerPanel extends FastPanel
   implements Constants, ActionListener
   {
   Debugger     debugger;
   
   // top level panels
   //
   ViewPanel    viewPanel;
   ControlPanel controlPanel;
   
   // innermost data panels
   //
   TextPanel    classesPanel;
   TextPanel    threadsPanel;
   TextPanel    framesPanel;
   TextPanel    localsPanel;
   TextPanel    inspectorPanel;
   ConsolePanel consolePanel;
   TextPanel    sourcePanel;
   MultiLabel   statusLabel;
   MessagePanel messagePanel;
   ButtonPanel  buttonPanel;
   
   DebuggerPanel(Debugger debugger)
      {
      this.debugger  = debugger;

      viewPanel      = new ViewPanel(debugger);
      controlPanel   = new ControlPanel(debugger);
      
      setLayout(new BorderLayout());
      add("Center", viewPanel);
      add("South",  controlPanel);
      
      classesPanel   = viewPanel.topPanel.classesPanel;
      threadsPanel   = viewPanel.topPanel.threadsPanel;
      framesPanel    = viewPanel.topPanel.framesPanel;
      localsPanel    = viewPanel.topPanel.localsPanel;
      inspectorPanel = viewPanel.insPanel.inspectorPanel;
      consolePanel   = viewPanel.conPanel.consolePanel;
      sourcePanel    = viewPanel.srcPanel.sourcePanel;

      messagePanel   = controlPanel.messagePanel;
      buttonPanel    = controlPanel.buttonPanel;

      getProperties();
      viewPanel.topPanel.adjustForComponentVisibilityChange();
      viewPanel.insPanel.adjustForComponentVisibilityChange();
      viewPanel.conPanel.adjustForComponentVisibilityChange();
      viewPanel.srcPanel.adjustForComponentVisibilityChange();
      viewPanel.adjustForComponentVisibilityChange();
      }

   // Make panels agree with property list settings.
   //
   synchronized
   void
   getProperties()
      {
      viewPanel.topPanel.classesPanel.setVisible(Session.getProperty(CLASSES_TITLE   + ".visible", "true").equals("true"));
      viewPanel.topPanel.threadsPanel.setVisible(Session.getProperty(THREADS_TITLE   + ".visible", "true").equals("true"));
      viewPanel.topPanel.framesPanel.setVisible( Session.getProperty(FRAMES_TITLE    + ".visible", "true").equals("true"));
      viewPanel.topPanel.localsPanel.setVisible( Session.getProperty(LOCALS_TITLE    + ".visible", "true").equals("true"));
      viewPanel.insPanel.setVisible(             Session.getProperty(INSPECTOR_TITLE + ".visible", "true").equals("true"));
      viewPanel.conPanel.setVisible(             Session.getProperty(CONSOLE_TITLE   + ".visible", "true").equals("true"));
      viewPanel.srcPanel.setVisible(             Session.getProperty(SOURCE_TITLE    + ".visible", "true").equals("true"));
      }

   // Make property list settings agree with panels.
   //
   synchronized
   void
   putProperties()
      {
      Session.putProperty(CLASSES_TITLE   + ".visible", "" + viewPanel.topPanel.classesPanel.isVisible());
      Session.putProperty(THREADS_TITLE   + ".visible", "" + viewPanel.topPanel.threadsPanel.isVisible());
      Session.putProperty(FRAMES_TITLE    + ".visible", "" + viewPanel.topPanel.framesPanel.isVisible());
      Session.putProperty(LOCALS_TITLE    + ".visible", "" + viewPanel.topPanel.localsPanel.isVisible());
      Session.putProperty(INSPECTOR_TITLE + ".visible", "" + viewPanel.insPanel.isVisible());
      Session.putProperty(CONSOLE_TITLE   + ".visible", "" + viewPanel.conPanel.isVisible());
      Session.putProperty(SOURCE_TITLE    + ".visible", "" + viewPanel.srcPanel.isVisible());
      }

   // Add relevant entries onto main window's menu bar.
   //
   synchronized
   void
   addMenu(MenuBar menuBar)
      {
      Menu     menu;
      MenuItem menuItem;
      
      menuBar.add(menu = new Menu("Views"));
      menu.add(menuItem = new MenuItem((viewPanel.topPanel.localsPanel.isVisible()  ? "+" : "-") + LOCALS_TITLE));    menuItem.addActionListener(this);
      menu.add(menuItem = new MenuItem((viewPanel.topPanel.framesPanel.isVisible()  ? "+" : "-") + FRAMES_TITLE));    menuItem.addActionListener(this);
      menu.add(menuItem = new MenuItem((viewPanel.topPanel.threadsPanel.isVisible() ? "+" : "-") + THREADS_TITLE));   menuItem.addActionListener(this);
      menu.add(menuItem = new MenuItem((viewPanel.topPanel.classesPanel.isVisible() ? "+" : "-") + CLASSES_TITLE));   menuItem.addActionListener(this);
      menu.add(menuItem = new MenuItem((viewPanel.insPanel.isVisible()              ? "+" : "-") + INSPECTOR_TITLE)); menuItem.addActionListener(this);
      menu.add(menuItem = new MenuItem((viewPanel.conPanel.isVisible()              ? "+" : "-") + CONSOLE_TITLE));   menuItem.addActionListener(this);
      menu.add(menuItem = new MenuItem((viewPanel.srcPanel.isVisible()              ? "+" : "-") + SOURCE_TITLE));    menuItem.addActionListener(this);

      menuBar.add(menu = new Menu("About"));
      menu.add(menuItem = new MenuItem(PROGRAM_TITLE)); menuItem.addActionListener(this);
      menu.add(menuItem = new MenuItem(AUTHOR_TITLE));  menuItem.addActionListener(this);
      }

   // Handle menubar hits.
   //
   synchronized
   public void
   actionPerformed(ActionEvent e) //- implements ActionListener
      {
      String cmd = e.getActionCommand();
      String panelTitle = cmd.substring(1);

      if (cmd.startsWith("+"))
         {
         ((MenuItem)e.getSource()).setLabel("-" + panelTitle);
         Session.frame.setBusyCursor();
         viewPanel.setTextPanelVisibility(panelTitle, false);
         Session.frame.setNormalCursor();
         }

      else if (cmd.startsWith("-"))
         {
         ((MenuItem)e.getSource()).setLabel("+" + panelTitle);
         viewPanel.setTextPanelVisibility(panelTitle, true);
         Session.eventQueue.put(new DebuggerEvent(DebuggerEvent.BUTTON_SNAPSHOT));
         }

      else if (cmd.equals(PROGRAM_TITLE))
         {
         String text = "\n This is jd, version " + VERSION + ".\n\n Jd is part of \"jikes\", a suite of free\n java program development tools produced by\n the Java Tools Group at IBM's Watson Research\n Center in Hawthorne NY. Jikes and other tools\n are available on the web at www.ibm.com/java/tools \n [click on \"IBM Technology Preview\"] and also\n at www.alphaworks.ibm.com/alphapreview_tools.\n\n Licensed Materials - Property of IBM \n (c) Copyright IBM Corp. 1997\n All Rights Reserved\n";
         MessageDialog dialog = new MessageDialog(Session.frame, "Program Information", text);
         dialog.edit();
         }
      
      else if (cmd.equals(AUTHOR_TITLE))
         {
         String text = "\n Derek Lieber (derek@watson.ibm.com) \n     IBM Watson Research Center\n      Hawthorne, NY, USA 10532\n";
         MessageDialog dialog = new MessageDialog(Session.frame, "Author Information", text);
         dialog.edit();
         }
      }
   }

class ViewPanel extends HorizontallySplitPanel
   implements Constants, AdjustableContainer
   {
   Panel    nulPanel;
   TopPanel topPanel;
   InsPanel insPanel;
   ConPanel conPanel;
   SrcPanel srcPanel;
   
   // zoom info
   //
   Container zoomParent; // zoomed panel's container (null --> nobody zoomed)
   int       zoomIndex;  // zoomed panel's z-order within that container
   
   ViewPanel(Debugger debugger)
      {
      nulPanel = new Panel();
      topPanel = new TopPanel(debugger);
      insPanel = new InsPanel(debugger);
      conPanel = new ConPanel(debugger);
      srcPanel = new SrcPanel(debugger);
      
      adjustForComponentVisibilityChange();
      }

   synchronized
   public void
   adjustForComponentVisibilityChange() //- implements AdjustableContainer
      {
      while (getComponentCount() != 0)
         remove(getComponent(0));

      if (nulPanel.isVisible())  add(nulPanel, "0");
      if (topPanel.isVisible())  add(topPanel, "1");
      if (insPanel.isVisible())  add(insPanel, "1");
      if (conPanel.isVisible())  add(conPanel, "1");
      if (srcPanel.isVisible())  add(srcPanel, "2");
      
      doLayout();
      }

   // Change subpanel visibility and reorganize viewing real estate.
   // Taken: subpanel's title
   //        its new visibility state
   //
   synchronized
   void 
   setTextPanelVisibility(String title, boolean isVisible)
      {
      setVisible(false);
      if (title.equals(CLASSES_TITLE))   { topPanel.classesPanel.setVisible(isVisible); topPanel.adjustForComponentVisibilityChange(); }
      if (title.equals(THREADS_TITLE))   { topPanel.threadsPanel.setVisible(isVisible); topPanel.adjustForComponentVisibilityChange(); }
      if (title.equals(FRAMES_TITLE))    { topPanel.framesPanel.setVisible(isVisible);  topPanel.adjustForComponentVisibilityChange(); }
      if (title.equals(LOCALS_TITLE))    { topPanel.localsPanel.setVisible(isVisible);  topPanel.adjustForComponentVisibilityChange(); }
      
      boolean isTopVisible = topPanel.classesPanel.isVisible() 
                          || topPanel.threadsPanel.isVisible() 
                          || topPanel.framesPanel.isVisible() 
                          || topPanel.localsPanel.isVisible();

                                         { topPanel.setVisible(isTopVisible); this.adjustForComponentVisibilityChange(); }
      if (title.equals(INSPECTOR_TITLE)) { insPanel.setVisible(isVisible);    this.adjustForComponentVisibilityChange(); }
      if (title.equals(CONSOLE_TITLE))   { conPanel.setVisible(isVisible);    this.adjustForComponentVisibilityChange(); }
      if (title.equals(SOURCE_TITLE))    { srcPanel.setVisible(isVisible);    this.adjustForComponentVisibilityChange(); }
      setVisible(true);
      }
   
   // Zoom subpanel to fill entire window, or restore subpanel's original size.
   //
   synchronized
   void
   zoom(Component target)
      {
      setVisible(false);
      
      if (zoomParent != null)
         { // unzoom by reparenting to original container and restoring original size
         zoomParent.add(target, zoomIndex);
         zoomParent = null;
         adjustForComponentVisibilityChange();
         }
      else
         { // zoom by reparenting to outer container and maximizing size
         zoomParent = target.getParent();
         for (int i = 0, n = zoomParent.getComponentCount(); i < n; ++i)
            if (zoomParent.getComponent(i) == target)
               zoomIndex = i;
         while (getComponentCount() != 0)
            remove(getComponent(0));
         add(nulPanel, "0");
         add(target,   "1");
         doLayout();
         }
      
      setVisible(true);
      }
   }

class TopPanel extends VerticallySplitPanel
   implements Constants, AdjustableContainer
   {
   TextPanel classesPanel;
   TextPanel threadsPanel;
   TextPanel framesPanel;
   TextPanel localsPanel;

   TopPanel(Debugger debugger)
      {
      localsPanel  = new TextPanel("locals",  LOCALS_TITLE,  debugger);
      framesPanel  = new TextPanel("callers", FRAMES_TITLE,  debugger);
      threadsPanel = new TextPanel("threads", THREADS_TITLE, debugger);
      classesPanel = new TextPanel("classes", CLASSES_TITLE, debugger);
      
      adjustForComponentVisibilityChange();
      }

   synchronized
   public void
   adjustForComponentVisibilityChange() //- implements AdjustableContainer
      {
      while (getComponentCount() != 0)
         remove(getComponent(0));

      if (localsPanel.isVisible())  add(localsPanel,  "1");
      if (framesPanel.isVisible())  add(framesPanel,  "1");
      if (threadsPanel.isVisible()) add(threadsPanel, "1");
      if (classesPanel.isVisible()) add(classesPanel, "1");
      
      doLayout();
      }
   }

class InsPanel extends VerticallySplitPanel
   implements Constants, AdjustableContainer
   {
   TextPanel inspectorPanel;
   
   InsPanel(Debugger debugger)
      {
      inspectorPanel = new TextPanel("inspector", INSPECTOR_TITLE, debugger);
      adjustForComponentVisibilityChange();
      }

   synchronized
   public void
   adjustForComponentVisibilityChange() //- implements AdjustableContainer
      {
      while (getComponentCount() != 0)
         remove(getComponent(0));
      if (inspectorPanel.isVisible()) add(inspectorPanel, "1");
      doLayout();
      }
   }

class ConPanel extends VerticallySplitPanel
   implements Constants, AdjustableContainer
   {
   ConsolePanel consolePanel;
   
   ConPanel(Debugger debugger)
      {
      consolePanel = new ConsolePanel(CONSOLE_TITLE);
      adjustForComponentVisibilityChange();
      }

   synchronized
   public void
   adjustForComponentVisibilityChange() //- implements AdjustableContainer
      {
      while (getComponentCount() != 0)
         remove(getComponent(0));
      if (consolePanel.isVisible()) add(consolePanel, "1");
      doLayout();
      }
   }

class SrcPanel extends VerticallySplitPanel
   implements Constants, AdjustableContainer
   {
   TextPanel sourcePanel;
   
   SrcPanel(Debugger debugger)
      {
      sourcePanel = new TextPanel("source", SOURCE_TITLE, debugger);
      adjustForComponentVisibilityChange();
      }

   synchronized
   public void
   adjustForComponentVisibilityChange() //- implements AdjustableContainer
      {
      while (getComponentCount() != 0)
         remove(getComponent(0));
      if (sourcePanel.isVisible()) add(sourcePanel, "1");
      doLayout();
      }
   }

class ControlPanel extends FastPanel
   implements Constants
   {
   MultiLabel   logoLabel;
   MessagePanel messagePanel;
   ButtonPanel  buttonPanel;
   
   ControlPanel(Debugger debugger)
      {
      logoLabel    = new MultiLabel(DebuggerFrame.logoImage, new Insets(0, 5, 0, 10));
      messagePanel = new MessagePanel();
      buttonPanel  = new ButtonPanel(debugger);

      GridBagLayout layoutManager = new GridBagLayout();

      setLayout(layoutManager);
      
      layoutManager.setConstraints(logoLabel, new FixedWidthConstraints(false));
      add(logoLabel);
      
      layoutManager.setConstraints(messagePanel, new ExpandableWidthConstraints(false));
      add(messagePanel);
      
      layoutManager.setConstraints(buttonPanel, new FixedWidthConstraints(false));
      add(buttonPanel);
      }
   }
