// Debugger session properties and shared data.
//
package jd;
import java.util.Properties;
import java.io.*;
import ui.*;

class Session
   implements Constants
   {
   private static Properties         properties;        // persistent data
           static DebuggerEventQueue eventQueue;        // delivery site for queuing up debugger and gui event reports
           static DebuggerFrame      frame;             // top level window to serve as parent for popups and flyovers
   private static FlyoverWindow      flyoverWindow;     // top level window for flyover help messages
   
   // Initialize.
   // Taken: session's event reporting queue
   //        session's top level window
   //
   synchronized
   static void
   init(DebuggerEventQueue q, DebuggerFrame f)
      {
      eventQueue     = q;
      frame          = f;
      flyoverWindow  = new FlyoverWindow(frame);
      }

   // Display flyover help message.
   //
   synchronized
   static void
   flyOn(String message, int screenX, int screenY)
      {
      flyoverWindow.on(message, screenX, screenY);
      }

   // Remove flyover help message.
   //
   synchronized
   static void
   flyOff()
      {
      flyoverWindow.off();
      }

   // Display informational message in popup dialog window.
   //
   static void
   popup(String message)
      {
      MessageDialog dialog = new MessageDialog(frame, "Error Information", " " + message + " ");
      dialog.edit();
      }

   // Display informational message on user's console.
   //
   synchronized
   static void
   warn(String message)
      {
      System.out.println("jd: " + message);
      }

   // Fetch a property.
   //
   synchronized
   static String
   getProperty(String key, String defaultValue)
      {
      if (properties == null)
         loadProperties();
      return properties.getProperty(key, defaultValue);
      }
   
   // Update a property.
   //
   synchronized
   static void
   putProperty(String key, String value)
      {
      if (properties == null)
         loadProperties();
      properties.put(key, value);
      }

   // Save property list to external file.
   //
   synchronized
   static void
   saveProperties()
      {
      if (properties == null)
         loadProperties();
      try
         {
         FileOutputStream fout = new FileOutputStream(PROPERTIES_FILENAME);
         properties.save(fout, "jd startup properties (you may erase this file if you wish)");
         fout.close();
         }
      catch(IOException e)
         {
         }
      }

   // Load property list from external file.
   //
   synchronized
   private static void
   loadProperties()
      {
      properties = new Properties();
      try
         {
         FileInputStream fin = new FileInputStream(PROPERTIES_FILENAME);
         properties.load(fin);
         fin.close();
         }
      catch(IOException e)
         {
         }
      if (!getProperty("version", "").equals(VERSION))
         { // property list from obsolete version - don't use it
         properties = new Properties();
         putProperty("version", VERSION);
         }
      }
   }
