// Base class for snapshots of remote data to be displayed in a local TextPanel.
// 11 Mar 1996 Derek Lieber
//
package jd;
import  ui.*;

class Snapshot
   {
   protected String text;         // lines of text to be displayed
   protected int    selectedLino; // line to appear hilighted, 0-indexed (-1 --> none)
   
   // Update contents of text panel to match this snapshot.
   // Taken: panel to update
   //        snapshot corresponding to when that panel was last updated (null --> none)
   //
   synchronized
   void 
   update(TextScroller body, Snapshot previousSnapshot)
      {
      int oldTop = body.getTopmostLine();

      // don't throw away old text unless we really need to
      //
      if (sameTextAs(previousSnapshot))
         body.undecorate();
      else
         body.setText(text);

      // choose center of view
      //
      if (sameViewpointAs(previousSnapshot))
         body.setTopmostLine(oldTop);      // desired text still in view: keep most recent viewpoint
      else
         body.setViewpoint(selectedLino);  // desired text might be out of view: change viewpoint to center it, if necessary

      // adjust hilighting
      //
      body.setMarkedLine(selectedLino);
      body.setSearchLine(selectedLino);
      }
      
   // Is text comprising "this" snapshot same as text comprising "previousSnapshot"?
   //
   synchronized
   boolean
   sameTextAs(Snapshot previousSnapshot)
      {
      return previousSnapshot != null && this.text.equals(previousSnapshot.text);
      }
   
   // Should "this" snapshot be displayed with same currently-visible-line as "previousSnapshot"?
   //
   synchronized
   boolean
   sameViewpointAs(Snapshot previousSnapshot)
      {
      return false;
      }
   }
