// Snapshot of a source file.
// 11 Mar 1996 Derek Lieber
//
package jd;
import  java.util.*;
import  java.io.*;
import  sun.tools.debug.*;
import  ui.*;
import  util.Tracer;

class SourceSnapshot extends Snapshot
   {
           String      packageName;           // name of package - something like "java.lang"       (null --> no source)
           String      remoteSourceName;      // name of source - something like "Object.java"      (null --> no source)
   private boolean     sourceAvailable;       // is source text available for display?
   private Vector      decorations;           // linenumbers to be decorated with breakpoint icons
   private boolean     centerViewAtSelection; // when displayed, should "selected" line appear at center of view?
   static final String SOURCE_UNAVAILABLE = "<source unavailable>\n";

   // This snapshot's currently selected linenumber (null --> no selection).
   //
   synchronized
   Integer
   currentSelection()
      {
      return selectedLino == -1 ? null : new Integer(selectedLino);
      }
   
   // Create snapshot of remote source file corresponding to PC of specified stackframe (iff non-null),
   // with "currentSelection" set to its corresponding line number (if any).
   // Runs in "Debugger" thread.
   //
   SourceSnapshot(SourceFinder sourceFinder, RemoteStackFrame remoteFrame, Vector breakpointList)
      throws Exception
      {
   // Tracer.trace("genText", "source");
      init(sourceFinder, remoteFrame == null ? null : remoteFrame.getRemoteClass());
      if (sourceAvailable)
         selectedLino = remoteFrame.getLineNumber() - 1;
      else
         selectedLino = -1;
      centerViewAtSelection = true;
      applyBreakpointDecorations(breakpointList);
      }

   // Create snapshot of remote source file for (possibly) different class.
   // Runs in "Debugger" thread.
   //
   SourceSnapshot(SourceFinder sourceFinder, RemoteClass remoteClass, Vector breakpointList)
      throws Exception
      {
   // Tracer.trace("genText", "source");
      init(sourceFinder, remoteClass);
      selectedLino = -1;
      centerViewAtSelection = true;
      applyBreakpointDecorations(breakpointList);
      }

   // Create copy of "oldSnapshot" with revised breakpoints.
   //
   SourceSnapshot(SourceSnapshot oldSnapshot, Vector breakpointList)
      throws Exception
      {
      text                  = oldSnapshot.text;
      selectedLino          = oldSnapshot.selectedLino;
      centerViewAtSelection = false;
      packageName           = oldSnapshot.packageName;
      remoteSourceName      = oldSnapshot.remoteSourceName;
      sourceAvailable       = oldSnapshot.sourceAvailable;
      applyBreakpointDecorations(breakpointList);
      }

   // Does this source file currently have a breakpoint set at specified line?
   //
   synchronized
   boolean
   hasBreakpointAt(Integer lino)
      {
      for (Enumeration e = decorations.elements(); e.hasMoreElements(); )
         if (((Integer)e.nextElement()).equals(lino))
            return true;
      return false;
      }

   // Load scroller with latest text and markup, just prior to painting screen.
   // Runs in "AWT" thread.
   //
   synchronized
   void 
   update(TextScroller body, Snapshot previousSnapshot) //- extends Snapshot
      {
      super.update(body, previousSnapshot);
      for (Enumeration e = decorations.elements(); e.hasMoreElements(); )
         body.setDecoratedLine(((Integer)e.nextElement()).intValue(), DebuggerFrame.stopSignImage);
      }

   // Faster version of superclass method (compares filenames instead of file contents).
   //
   synchronized
   boolean
   sameTextAs(Snapshot s) //- overrides Snapshot
      {
      SourceSnapshot that = (SourceSnapshot)s;

      return that != null                              &&

             this.packageName != null                  &&
             that.packageName != null                  &&

             this.remoteSourceName != null             &&
             that.remoteSourceName != null             &&
      
             this.packageName.equals(that.packageName) &&
             this.remoteSourceName.equals(that.remoteSourceName);
      }

   synchronized
   boolean
   sameViewpointAs(Snapshot s) //- overrides Snapshot
      {
      return !centerViewAtSelection;
      }
      
   // Force next call of init() to reread filesystem.
   //
   synchronized
   void
   invalidate()
      {
      packageName      = "?";
      remoteSourceName = "?";
      sourceAvailable  = false;
      text             = "?\n";
      }
      
   // Fetch a class's source file text.
   // Taken: places to look
   //        any class that has this file as a source
   //
   synchronized
   private void
   init(SourceFinder sourceFinder, RemoteClass remoteClass)
      throws Exception
      {
      if (remoteClass == null)
         {
         packageName      = null;
         remoteSourceName = null;
         sourceAvailable  = false;
         text             = "";
      // System.out.println("source: null file");
         return;
         }

      String prevSourceName  = remoteSourceName;
      String prevPackageName = packageName;
      
      packageName      = ClassFinder.packageForClass(remoteClass.getName());
      remoteSourceName = remoteClass.getSourceFileName();

      if (prevSourceName  != null && prevSourceName.equals(remoteSourceName) &&
          prevPackageName != null && prevPackageName.equals(packageName))
         { // same source file as what we're currently holding - don't waste time fetching it again
      // System.out.println("source: same file");
         return;
         }

   // // ask remote debugger for source text
   // //
   // InputStream input = remoteClass.getSourceFile();
   // if (input != null)
   //    {
   //    byte data[] = new byte[input.available()];
   //    input.read(data);
   //    input.close();
   //    text            = TextScroller.stringFromBytes(data);
   //    sourceAvailable = true;
   //    return;
   //    }

      byte[] data = sourceFinder.getFileContents(packageName, remoteSourceName);
      if (data != null)
         {
         text            = TextScroller.stringFromBytes(data);
         sourceAvailable = true;
      // System.out.println("source: new file");
         return;
         }

      // not found
      //
      text            = SOURCE_UNAVAILABLE;
      sourceAvailable = false;
   // System.out.println("source: no file");
      }

   // Spawn editor on (local filesystem copy of) current source file.
   //
   synchronized
   void
   edit(SourceFinder sourceFinder)
      {
      if (!sourceAvailable)
         {
         Session.popup("Source file \"" + remoteSourceName + "\" not available");
         return;
         }

      String sourceFileName = sourceFinder.resolveSourceFileName(packageName, remoteSourceName);
      if (sourceFileName == null)
         {
         Session.popup("Source file \"" + remoteSourceName + "\" not found");
         return;
         }

      Editor.edit(sourceFileName, selectedLino + 1);
      }

   // Extract from "breakpointList" those breakpoints that correspond to this source file.
   //
   synchronized
   private void
   applyBreakpointDecorations(Vector breakpointList)
      {
      decorations = new Vector();
      
      if (!sourceAvailable)
         return;

      for (Enumeration b = breakpointList.elements(); b.hasMoreElements(); )
         {
         Breakpoint bp = (Breakpoint)b.nextElement();
         if (bp.sourceFileName.equals(remoteSourceName))
            decorations.addElement(new Integer(bp.lineNumber - 1));
         }
      }
   }
