// Edit a string within a modal popup window.
// See also: PathDialog
// 29 Sep 1997 Derek Lieber
//
package jd;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import ui.MultiButton;

class StringDialog extends Dialog
   implements Constants
   {
   private Frame     frame;
   private TextArea  textArea;
   private Container buttonPanel;
   private String    result;

   // Create popup editor.
   // Taken: frame relative to which we should pop up
   //        string to display in dialog's title bar
   //        initial value of string to be edited
   //
   StringDialog(Frame _frame, String title, String initialString)
      {
      super(_frame, title, true);

      frame       = _frame;
      textArea    =  new TextArea("", 1, 40, TextArea.SCROLLBARS_HORIZONTAL_ONLY);
      buttonPanel =  new Panel();
      result      =  null;

      setLayout(new BorderLayout());
      add("Center", textArea);
      add("South",  buttonPanel);
      
      MultiButton okButton     = new MultiButton(" Ok ", null);
      MultiButton cancelButton = new MultiButton(" Cancel ", null);
      
      textArea.addKeyListener(new KeyAdapter() { public void keyTyped(KeyEvent e) { if (e.getKeyChar() == '\r') ok(); }});
      okButton.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { ok(); }});
      cancelButton.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { cancel(); }});
      addWindowListener(new WindowAdapter() { public void windowClosing(WindowEvent e) { cancel(); }});

      buttonPanel.setLayout(new FlowLayout(FlowLayout.CENTER));
      buttonPanel.add("", okButton);
      buttonPanel.add("", cancelButton);

      textArea.setEditable(true);
      textArea.setFont(MONOSPACED_FONT);
      textArea.setText(initialString);
      textArea.select(initialString.length(), initialString.length()); // set dialog cursor to end of string
      }

   // Allow string to be edited.
   // Taken:    nothing
   // Returned: revised string (null --> "cancel" was hit)
   // Note: to avoid a deadlock between Dialog.show() and Dialog.ok(), this method must NOT be synchronized!
   //
// synchronized
   String
   edit()
      {
      pack();

      Point     frameLoc   = frame.getLocationOnScreen();
      Dimension frameSize  = frame.getSize();
      Dimension dialogSize = getPreferredSize();
      
      int x = frameLoc.x + frameSize.width  / 2 - dialogSize.width  / 2;
      int y = frameLoc.y + frameSize.height / 2 - dialogSize.height / 2;
      
      if (x < 0) x = 0;
      if (y < 0) y = 0;

      setBounds(x, y, dialogSize.width, dialogSize.height);
      show(); // note: dialog is "modal", meaning that control blocks here until ok() or cancel() hides the window again
      dispose();

      return result;
      }

   synchronized
   void
   ok()
      {
      result = textArea.getText();

      // AWT bug workaround: prevent desktop from getting focus when dialog vanishes
      //
      frame.toFront();      
      Thread.currentThread().yield();
      
      // unblock caller at site of "dialog.show()"
      //
      setVisible(false);
      }

   synchronized
   void
   cancel()
      {
      result = null;
      
      // AWT bug workaround: prevent desktop from getting focus when dialog vanishes
      //
      frame.toFront();      
      Thread.currentThread().yield();
      
      // unblock caller at site of "dialog.show()"
      //
      setVisible(false);
      }
   }
