// The standard AWT library generates unnecessary paint() requests as a side effect
// of setBounds(), add(), and remove(). This leads to excessive screen redrawing.
//
// This "fast" component class suppresses such behavior, in order to orthogonalize
// the operations of window shaping vs. drawing. The advantage is better screen
// performance. The disadvantage is that subclasses must decide for themselves when 
// to redraw the screen.
//
// See also: FastContainer, FastPanel
//
// 24 Sep 1997 Derek Lieber
//
package ui;
import java.awt.*;

public class FastComponent extends Component
   {
   public void
   update(Graphics g) //- overrides Component
      {
   // System.out.println("ignoring component update on " + getName());
      }

   public void
   repaint(long tm, int x, int y, int width, int height) //- overrides Component
      {
   // System.out.println("ignoring component repaint on " + getName());
      }
   }
