// Sorting functions.
// 15 Feb 1996 Derek Lieber.
//
package util;

import java.util.Vector;

// Shell sort.
// Reference: "The C Programming Language", Kernighan & Ritchie, p. 116
//
public class Sorter
   {
   // Sort elements of "x" using comparison function "cmp".
   //
   public static void
   sort(Vector x, Comparer cmp)
      {
      int n = x.size();
      for (int gap = n / 2; gap > 0; gap /= 2)
         for (int i = gap; i < n; ++i)
            for (int j = i - gap; j >= 0; j -= gap)
               {
               if (cmp.isLessThan(x.elementAt(j), x.elementAt(j + gap)))
                  break;
               swap(x, j, j + gap);
               }
      }
      
   // Interchange "x[i]" with "x[j]"
   //
   private static void
   swap(Vector x, int i, int j)
      {
      Object tmp = x.elementAt(i);
      x.setElementAt(x.elementAt(j), i);
      x.setElementAt(tmp, j);
      }
   }

/***
// Bubble sort.
//
public class Sorter
   {
   // Sort elements of "x" using comparison function "cmp".
   //
   public static void
   sort(Vector x, Comparer cmp)
      {
      boolean changed;
      do {
         changed = false;
         for (int i = 0; i < x.size() - 1; ++i)
            if (cmp.isLessThan(x.elementAt(i + 1), x.elementAt(i)))
               {
               swap(x, i + 1, i);
               changed = true;
               }
         } while (changed);
      }
      
   // Interchange "x[i]" with "x[j]"
   //
   private static void
   swap(Vector x, int i, int j)
      {
      Object tmp = x.elementAt(i);
      x.setElementAt(x.elementAt(j), i);
      x.setElementAt(tmp, j);
      }
   }
***/

/***
// Quick sort.
// Reference: "Data Structures", Reingold & Hansen, p. 385
//
public class Sorter
   {
   // Sort elements of "x" using comparison function "cmp".
   //
   public static void
   sort(Vector x, Comparer cmp)
      {
      if (x.size() < 2) return;
      
      // move greatest element last
      int greatest = 0;
      for (int i = 1; i < x.size(); ++i)
         if (cmp.isGreaterThan(x.elementAt(i), x.elementAt(greatest)))
            greatest = i;
      swap(x, greatest, x.size() - 1);
      
      // now sort
      sort(x, cmp, 0, x.size() - 2);
      }
      
   // Sort elements "x[first]...x[last]".
   // Assumption: "x[last + 1]" must exist and must be greater than or equal to "x[first]...x[last]".
   //
   private static void
   sort(Vector x, Comparer cmp, int first, int last)
      {
      if (first >= last)
         return;
         
      // partition the elements
      
      int i = first + 1;
      while (cmp.isLessThan(x.elementAt(i), x.elementAt(first)))
         ++i;

      int j = last;
      while (cmp.isGreaterThan(x.elementAt(j), x.elementAt(first)))
         --j;

      while (i < j)
         { 
         // at this point i < j and x[i] >= x[first] >= x[j]
         
         swap(x, i, j);
         
         while (cmp.isGreaterThan(x.elementAt(i), x.elementAt(first)))
            ++i;
         while (cmp.isLessThan(x.elementAt(j), x.elementAt(first)))
            --j;
         }
         
      swap(x, first, j);
      sort(x, cmp, first, j - 1);
      sort(x, cmp, j + 1, last);
      }

   // Interchange "x[i]" with "x[j]"
   //
   private static void
   swap(Vector x, int i, int j)
      {
      Object tmp = x.elementAt(i);
      x.setElementAt(x.elementAt(j), i);
      x.setElementAt(tmp, j);
      }
   }
***/
